@@ -15,9 +15,7 @@ use crate::{
     },
     theme::{Alignment, CodeBlockStyle},
 };
-use schemars::JsonSchema;
 use serde::Deserialize;
-use serde_with::DeserializeFromStr;
 use std::{cell::RefCell, convert::Infallible, fmt::Write, ops::Range, path::PathBuf, rc::Rc, str::FromStr};
 use strum::{EnumDiscriminants, EnumIter};
 use unicode_width::UnicodeWidthStr;
@@ -440,7 +438,8 @@ impl Snippet {
 }
 
 /// The language of a code snippet.
-#[derive(Clone, Debug, PartialEq, Eq, EnumIter, PartialOrd, Ord, DeserializeFromStr, JsonSchema)]
+#[derive(Clone, Debug, PartialEq, Eq, EnumIter, PartialOrd, Ord)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
 pub enum SnippetLanguage {
     Ada,
     Asp,
@@ -470,6 +469,7 @@ pub enum SnippetLanguage {
     Java,
     JavaScript,
     Json,
+    Julia,
     Kotlin,
     Latex,
     Lua,
@@ -508,6 +508,8 @@ pub enum SnippetLanguage {
     Zsh,
 }
 
+crate::utils::impl_deserialize_from_str!(SnippetLanguage);
+
 impl FromStr for SnippetLanguage {
     type Err = Infallible;
 
@@ -541,6 +543,7 @@ impl FromStr for SnippetLanguage {
             "java" => Java,
             "javascript" | "js" => JavaScript,
             "json" => Json,
+            "julia" => Julia,
             "kotlin" => Kotlin,
             "latex" => Latex,
             "lua" => Lua,
@@ -656,6 +659,8 @@ pub(crate) enum Highlight {
 pub(crate) struct ExternalFile {
     pub(crate) path: PathBuf,
     pub(crate) language: SnippetLanguage,
+    pub(crate) start_line: Option<usize>,
+    pub(crate) end_line: Option<usize>,
 }
 
 #[cfg(test)]
