@@ -7,7 +7,6 @@ use crate::{
     },
 };
 use clap::ValueEnum;
-use schemars::JsonSchema;
 use serde::Deserialize;
 use std::{
     collections::{BTreeMap, HashMap},
@@ -16,7 +15,8 @@ use std::{
     path::Path,
 };
 
-#[derive(Clone, Debug, Default, Deserialize, JsonSchema)]
+#[derive(Clone, Debug, Default, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
 #[serde(deny_unknown_fields)]
 pub struct Config {
     /// The default configuration for the presentation.
@@ -43,6 +43,9 @@ pub struct Config {
 
     #[serde(default)]
     pub export: ExportConfig,
+
+    #[serde(default)]
+    pub transition: Option<SlideTransitionConfig>,
 }
 
 impl Config {
@@ -70,7 +73,8 @@ pub enum ConfigLoadError {
     Invalid(#[from] serde_yaml::Error),
 }
 
-#[derive(Clone, Debug, Deserialize, JsonSchema)]
+#[derive(Clone, Debug, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
 #[serde(deny_unknown_fields)]
 pub struct DefaultsConfig {
     /// The theme to use by default in every presentation unless overridden.
@@ -78,7 +82,7 @@ pub struct DefaultsConfig {
 
     /// Override the terminal font size when in windows or when using sixel.
     #[serde(default = "default_terminal_font_size")]
-    #[validate(range(min = 1))]
+    #[cfg_attr(feature = "json-schema", validate(range(min = 1)))]
     pub terminal_font_size: u8,
 
     /// The image protocol to use.
@@ -90,7 +94,7 @@ pub struct DefaultsConfig {
     pub validate_overflows: ValidateOverflows,
 
     /// A max width in columns that the presentation must always be capped to.
-    #[serde(default = "default_max_columns")]
+    #[serde(default = "default_u16_max")]
     pub max_columns: u16,
 
     /// The alignment the presentation should have if `max_columns` is set and the terminal is
@@ -98,6 +102,15 @@ pub struct DefaultsConfig {
     #[serde(default)]
     pub max_columns_alignment: MaxColumnsAlignment,
 
+    /// A max height in rows that the presentation must always be capped to.
+    #[serde(default = "default_u16_max")]
+    pub max_rows: u16,
+
+    /// The alignment the presentation should have if `max_rows` is set and the terminal is
+    /// larger than that.
+    #[serde(default)]
+    pub max_rows_alignment: MaxRowsAlignment,
+
     /// The configuration for lists when incremental lists are enabled.
     #[serde(default)]
     pub incremental_lists: IncrementalListsConfig,
@@ -110,15 +123,18 @@ impl Default for DefaultsConfig {
             terminal_font_size: default_terminal_font_size(),
             image_protocol: Default::default(),
             validate_overflows: Default::default(),
-            max_columns: default_max_columns(),
+            max_columns: default_u16_max(),
             max_columns_alignment: Default::default(),
+            max_rows: default_u16_max(),
+            max_rows_alignment: Default::default(),
             incremental_lists: Default::default(),
         }
     }
 }
 
 /// The configuration for lists when incremental lists are enabled.
-#[derive(Clone, Debug, Default, Deserialize, JsonSchema)]
+#[derive(Clone, Debug, Default, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
 #[serde(deny_unknown_fields)]
 pub struct IncrementalListsConfig {
     /// Whether to pause before a list begins.
@@ -135,7 +151,8 @@ fn default_terminal_font_size() -> u8 {
 }
 
 /// The alignment to use when `defaults.max_columns` is set.
-#[derive(Clone, Copy, Debug, Default, Deserialize, JsonSchema)]
+#[derive(Clone, Copy, Debug, Default, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
 #[serde(rename_all = "snake_case")]
 pub enum MaxColumnsAlignment {
     /// Align the presentation to the left.
@@ -149,7 +166,24 @@ pub enum MaxColumnsAlignment {
     Right,
 }
 
-#[derive(Clone, Debug, Default, Deserialize, JsonSchema)]
+/// The alignment to use when `defaults.max_rows` is set.
+#[derive(Clone, Copy, Debug, Default, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
+#[serde(rename_all = "snake_case")]
+pub enum MaxRowsAlignment {
+    /// Align the presentation to the top.
+    Top,
+
+    /// Align the presentation on the center.
+    #[default]
+    Center,
+
+    /// Align the presentation to the bottom.
+    Bottom,
+}
+
+#[derive(Clone, Debug, Default, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
 #[serde(rename_all = "snake_case")]
 pub enum ValidateOverflows {
     #[default]
@@ -159,7 +193,8 @@ pub enum ValidateOverflows {
     WhenDeveloping,
 }
 
-#[derive(Clone, Debug, Default, Deserialize, JsonSchema)]
+#[derive(Clone, Debug, Default, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
 #[serde(deny_unknown_fields)]
 pub struct OptionsConfig {
     /// Whether slides are automatically terminated when a slide title is found.
@@ -185,7 +220,8 @@ pub struct OptionsConfig {
     pub auto_render_languages: Vec<SnippetLanguage>,
 }
 
-#[derive(Clone, Debug, Default, Deserialize, JsonSchema)]
+#[derive(Clone, Debug, Default, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
 #[serde(deny_unknown_fields)]
 pub struct SnippetConfig {
     /// The properties for snippet execution.
@@ -201,7 +237,8 @@ pub struct SnippetConfig {
     pub render: SnippetRenderConfig,
 }
 
-#[derive(Clone, Debug, Default, Deserialize, JsonSchema)]
+#[derive(Clone, Debug, Default, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
 #[serde(deny_unknown_fields)]
 pub struct SnippetExecConfig {
     /// Whether to enable snippet execution.
@@ -212,7 +249,8 @@ pub struct SnippetExecConfig {
     pub custom: BTreeMap<SnippetLanguage, LanguageSnippetExecutionConfig>,
 }
 
-#[derive(Clone, Debug, Default, Deserialize, JsonSchema)]
+#[derive(Clone, Debug, Default, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
 #[serde(deny_unknown_fields)]
 pub struct SnippetExecReplaceConfig {
     /// Whether to enable snippet replace-executions, which automatically run code snippets without
@@ -220,7 +258,8 @@ pub struct SnippetExecReplaceConfig {
     pub enable: bool,
 }
 
-#[derive(Clone, Debug, Deserialize, JsonSchema)]
+#[derive(Clone, Debug, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
 #[serde(deny_unknown_fields)]
 pub struct SnippetRenderConfig {
     /// The number of threads to use when rendering.
@@ -238,7 +277,8 @@ pub(crate) fn default_snippet_render_threads() -> usize {
     2
 }
 
-#[derive(Clone, Debug, Deserialize, JsonSchema)]
+#[derive(Clone, Debug, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
 #[serde(deny_unknown_fields)]
 pub struct TypstConfig {
     /// The pixels per inch when rendering latex/typst formulas.
@@ -256,7 +296,8 @@ pub(crate) fn default_typst_ppi() -> u32 {
     300
 }
 
-#[derive(Clone, Debug, Deserialize, JsonSchema)]
+#[derive(Clone, Debug, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
 #[serde(deny_unknown_fields)]
 pub struct MermaidConfig {
     /// The scaling parameter to be used in the mermaid CLI.
@@ -274,12 +315,13 @@ pub(crate) fn default_mermaid_scale() -> u32 {
     2
 }
 
-pub(crate) fn default_max_columns() -> u16 {
+pub(crate) fn default_u16_max() -> u16 {
     u16::MAX
 }
 
 /// The snippet execution configuration for a specific programming language.
-#[derive(Clone, Debug, Deserialize, JsonSchema)]
+#[derive(Clone, Debug, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
 pub struct LanguageSnippetExecutionConfig {
     /// The filename to use for the snippet input file.
     pub filename: String,
@@ -295,7 +337,8 @@ pub struct LanguageSnippetExecutionConfig {
     pub hidden_line_prefix: Option<String>,
 }
 
-#[derive(Clone, Debug, Default, Deserialize, ValueEnum, JsonSchema)]
+#[derive(Clone, Debug, Default, Deserialize, ValueEnum)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
 #[serde(rename_all = "kebab-case")]
 pub enum ImageProtocol {
     /// Automatically detect the best image protocol to use.
@@ -349,7 +392,8 @@ impl TryFrom<&ImageProtocol> for GraphicsMode {
     }
 }
 
-#[derive(Clone, Debug, Deserialize, JsonSchema)]
+#[derive(Clone, Debug, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
 #[serde(deny_unknown_fields)]
 pub struct KeyBindingsConfig {
     /// The keys that cause the presentation to move forwards.
@@ -436,7 +480,8 @@ impl Default for KeyBindingsConfig {
     }
 }
 
-#[derive(Clone, Debug, Deserialize, JsonSchema)]
+#[derive(Clone, Debug, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
 #[serde(deny_unknown_fields)]
 pub struct SpeakerNotesConfig {
     /// The address in which to listen for speaker note events.
@@ -463,15 +508,34 @@ impl Default for SpeakerNotesConfig {
 }
 
 /// The export configuration.
-#[derive(Clone, Debug, Default, Deserialize, JsonSchema)]
+#[derive(Clone, Debug, Default, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
 #[serde(deny_unknown_fields)]
 pub struct ExportConfig {
     /// The dimensions to use for presentation exports.
     pub dimensions: Option<ExportDimensionsConfig>,
+
+    /// Whether pauses should create new slides.
+    #[serde(default)]
+    pub pauses: PauseExportPolicy,
+}
+
+/// The policy for pauses when exporting.
+#[derive(Clone, Debug, Default, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
+#[serde(deny_unknown_fields, rename_all = "snake_case")]
+pub enum PauseExportPolicy {
+    /// Whether to ignore pauses.
+    #[default]
+    Ignore,
+
+    /// Create a new slide when a pause is found.
+    NewSlide,
 }
 
 /// The dimensions to use for presentation exports.
-#[derive(Clone, Debug, Deserialize, JsonSchema)]
+#[derive(Clone, Debug, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
 #[serde(deny_unknown_fields)]
 pub struct ExportDimensionsConfig {
     /// The number of rows.
@@ -481,6 +545,38 @@ pub struct ExportDimensionsConfig {
     pub columns: u16,
 }
 
+// The slide transition configuration.
+#[derive(Clone, Debug, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
+#[serde(tag = "style", deny_unknown_fields)]
+pub struct SlideTransitionConfig {
+    /// The amount of time to take to perform the transition.
+    #[serde(default = "default_transition_duration_millis")]
+    pub duration_millis: u16,
+
+    /// The number of frames in a transition.
+    #[serde(default = "default_transition_frames")]
+    pub frames: usize,
+
+    /// The slide transition style.
+    pub animation: SlideTransitionStyleConfig,
+}
+
+// The slide transition style configuration.
+#[derive(Clone, Debug, Deserialize)]
+#[cfg_attr(feature = "json-schema", derive(schemars::JsonSchema))]
+#[serde(tag = "style", rename_all = "snake_case", deny_unknown_fields)]
+pub enum SlideTransitionStyleConfig {
+    /// Slide horizontally.
+    SlideHorizontal,
+
+    /// Fade the new slide into the previous one.
+    Fade,
+
+    /// Collapse the current slide into the center of the screen.
+    CollapseHorizontal,
+}
+
 fn make_keybindings<const N: usize>(raw_bindings: [&str; N]) -> Vec<KeyBinding> {
     let mut bindings = Vec::new();
     for binding in raw_bindings {
@@ -545,6 +641,14 @@ fn default_suspend_bindings() -> Vec<KeyBinding> {
     make_keybindings(["<c-z>"])
 }
 
+fn default_transition_duration_millis() -> u16 {
+    1000
+}
+
+fn default_transition_frames() -> usize {
+    30
+}
+
 #[cfg(target_os = "linux")]
 pub(crate) fn default_speaker_notes_listen_address() -> SocketAddr {
     SocketAddr::new(IpAddr::V4(Ipv4Addr::new(127, 255, 255, 255)), 59418)
