@@ -26,12 +26,11 @@ use crossterm::{
 };
 use directories::ProjectDirs;
 use export::exporter::OutputDirectory;
-use render::properties::WindowSize;
+use render::{engine::MaxSize, properties::WindowSize};
 use std::{
     env::{self, current_dir},
     io,
     path::{Path, PathBuf},
-    rc::Rc,
     sync::Arc,
 };
 use terminal::emulator::TerminalEmulator;
@@ -51,7 +50,9 @@ mod terminal;
 mod theme;
 mod third_party;
 mod tools;
+mod transitions;
 mod ui;
+mod utils;
 
 const DEFAULT_THEME: &str = "dark";
 const DEFAULT_EXPORT_PIXELS_PER_COLUMN: u16 = 20;
@@ -74,8 +75,13 @@ struct Cli {
     #[clap(long, requires = "export_pdf")]
     export_temporary_path: Option<PathBuf>,
 
+    /// The output path for the exported PDF.
+    #[clap(short = 'o', long = "output", requires = "export_pdf")]
+    export_output: Option<PathBuf>,
+
     /// Generate a JSON schema for the configuration file.
     #[clap(long)]
+    #[cfg(feature = "json-schema")]
     generate_config_file_schema: bool,
 
     /// Use presentation mode.
@@ -187,7 +193,7 @@ impl Customizations {
 
 struct CoreComponents {
     third_party: ThirdPartyRender,
-    code_executor: Rc<SnippetExecutor>,
+    code_executor: Arc<SnippetExecutor>,
     resources: Resources,
     printer: Arc<ImagePrinter>,
     builder_options: PresentationBuilderOptions,
@@ -225,7 +231,7 @@ impl CoreComponents {
         }
         let graphics_mode = Self::select_graphics_mode(cli, &config);
         let printer = Arc::new(ImagePrinter::new(graphics_mode.clone())?);
-        let registry = ImageRegistry(printer.clone());
+        let registry = ImageRegistry::new(printer.clone());
         let resources = Resources::new(
             resources_path.clone(),
             themes_path.unwrap_or_else(|| resources_path.clone()),
@@ -237,7 +243,7 @@ impl CoreComponents {
             threads: config.snippet.render.threads,
         };
         let third_party = ThirdPartyRender::new(third_party_config, registry, &resources_path);
-        let code_executor = Rc::new(code_executor);
+        let code_executor = Arc::new(code_executor);
         Ok(Self {
             third_party,
             code_executor,
@@ -278,6 +284,7 @@ impl CoreComponents {
             theme_options: ThemeOptions { font_size_supported: TerminalEmulator::capabilities().font_size },
             pause_before_incremental_lists: config.defaults.incremental_lists.pause_before.unwrap_or(true),
             pause_after_incremental_lists: config.defaults.incremental_lists.pause_after.unwrap_or(true),
+            pause_create_new_slide: false,
         }
     }
 
@@ -343,11 +350,13 @@ fn overflow_validation_enabled(mode: &PresentMode, config: &ValidateOverflows) -
 }
 
 fn run(cli: Cli) -> Result<(), Box<dyn std::error::Error>> {
+    #[cfg(feature = "json-schema")]
     if cli.generate_config_file_schema {
         let schema = schemars::schema_for!(Config);
         serde_json::to_writer_pretty(io::stdout(), &schema).map_err(|e| format!("failed to write schema: {e}"))?;
         return Ok(());
-    } else if cli.acknowledgements {
+    }
+    if cli.acknowledgements {
         let acknowledgements = include_bytes!("../bat/acknowledgements.txt");
         println!("{}", String::from_utf8_lossy(acknowledgements));
         return Ok(());
@@ -411,12 +420,13 @@ fn run(cli: Cli) -> Result<(), Box<dyn std::error::Error>> {
             themes,
             builder_options,
             dimensions,
+            config.export.pauses,
         );
         let output_directory = match cli.export_temporary_path {
             Some(path) => OutputDirectory::external(path),
             None => OutputDirectory::temporary(),
         }?;
-        exporter.export_pdf(&path, output_directory)?;
+        exporter.export_pdf(&path, output_directory, cli.export_output.as_deref())?;
     } else {
         let SpeakerNotesComponents { events_listener, events_publisher } =
             SpeakerNotesComponents::new(&cli, &config, &path)?;
@@ -429,8 +439,13 @@ fn run(cli: Cli) -> Result<(), Box<dyn std::error::Error>> {
             font_size_fallback: config.defaults.terminal_font_size,
             bindings: config.bindings,
             validate_overflows,
-            max_columns: config.defaults.max_columns,
-            max_columns_alignment: config.defaults.max_columns_alignment,
+            max_size: MaxSize {
+                max_columns: config.defaults.max_columns,
+                max_columns_alignment: config.defaults.max_columns_alignment,
+                max_rows: config.defaults.max_rows,
+                max_rows_alignment: config.defaults.max_rows_alignment,
+            },
+            transition: config.transition,
         };
         let presenter = Presenter::new(
             &default_theme,
