@@ -22,7 +22,7 @@ use crate::{
     resource::Resources,
     terminal::image::{
         Image,
-        printer::{ImageRegistry, RegisterImageError},
+        printer::{ImageRegistry, ImageSpec, RegisterImageError},
     },
     theme::{
         Alignment, AuthorPositioning, ElementType, PresentationTheme, ProcessingThemeError, ThemeOptions,
@@ -40,7 +40,7 @@ use comrak::{Arena, nodes::AlertType};
 use image::DynamicImage;
 use serde::Deserialize;
 use snippet::{SnippetOperations, SnippetProcessor, SnippetProcessorState};
-use std::{collections::HashSet, fmt::Display, iter, mem, path::PathBuf, rc::Rc, str::FromStr};
+use std::{collections::HashSet, fmt::Display, iter, mem, path::PathBuf, rc::Rc, str::FromStr, sync::Arc};
 use unicode_width::UnicodeWidthStr;
 
 mod snippet;
@@ -71,6 +71,7 @@ pub struct PresentationBuilderOptions {
     pub theme_options: ThemeOptions,
     pub pause_before_incremental_lists: bool,
     pub pause_after_incremental_lists: bool,
+    pub pause_create_new_slide: bool,
 }
 
 impl PresentationBuilderOptions {
@@ -111,6 +112,7 @@ impl Default for PresentationBuilderOptions {
             theme_options: ThemeOptions { font_size_supported: false },
             pause_before_incremental_lists: true,
             pause_after_incremental_lists: true,
+            pause_create_new_slide: false,
         }
     }
 }
@@ -125,7 +127,7 @@ pub(crate) struct PresentationBuilder<'a> {
     chunk_mutators: Vec<Box<dyn ChunkMutator>>,
     slide_builders: Vec<SlideBuilder>,
     highlighter: SnippetHighlighter,
-    code_executor: Rc<SnippetExecutor>,
+    code_executor: Arc<SnippetExecutor>,
     theme: PresentationTheme,
     default_raw_theme: &'a raw::PresentationTheme,
     resources: Resources,
@@ -148,7 +150,7 @@ impl<'a> PresentationBuilder<'a> {
         default_raw_theme: &'a raw::PresentationTheme,
         resources: Resources,
         third_party: &'a mut ThirdPartyRender,
-        code_executor: Rc<SnippetExecutor>,
+        code_executor: Arc<SnippetExecutor>,
         themes: &'a Themes,
         image_registry: ImageRegistry,
         bindings_config: KeyBindingsConfig,
@@ -246,7 +248,7 @@ impl<'a> PresentationBuilder<'a> {
         };
         let mut image = DynamicImage::new_rgba8(1, 1);
         image.as_mut_rgba8().unwrap().get_pixel_mut(0, 0).0 = rgba;
-        let image = self.image_registry.register_image(image)?;
+        let image = self.image_registry.register(ImageSpec::Generated(image))?;
         Ok(image)
     }
 
@@ -377,7 +379,16 @@ impl<'a> PresentationBuilder<'a> {
                 new_theme = Some(theme);
             }
             if let Some(theme_path) = &metadata.path {
-                let theme = self.resources.theme(theme_path)?;
+                let mut theme = self.resources.theme(theme_path)?;
+                if let Some(name) = &theme.extends {
+                    let base = self
+                        .themes
+                        .presentation
+                        .load_by_name(name)
+                        .ok_or_else(|| BuildError::InvalidMetadata(format!("extended theme {name} not found")))?;
+                    theme = merge_struct::merge(&theme, &base)
+                        .map_err(|e| BuildError::InvalidMetadata(format!("invalid theme: {e}")))?;
+                }
                 new_theme = Some(theme);
             }
         }
@@ -385,8 +396,9 @@ impl<'a> PresentationBuilder<'a> {
             if overrides.extends.is_some() {
                 return Err(BuildError::InvalidMetadata("theme overrides can't use 'extends'".into()));
             }
+            let base = new_theme.as_ref().unwrap_or(self.default_raw_theme);
             // This shouldn't fail as the models are already correct.
-            let theme = merge_struct::merge(self.default_raw_theme, overrides)
+            let theme = merge_struct::merge(base, overrides)
                 .map_err(|e| BuildError::InvalidMetadata(format!("invalid theme: {e}")))?;
             new_theme = Some(theme);
         }
@@ -598,6 +610,12 @@ impl<'a> PresentationBuilder<'a> {
     }
 
     fn push_pause(&mut self) {
+        if self.options.pause_create_new_slide {
+            let operations = self.chunk_operations.clone();
+            self.terminate_slide();
+            self.chunk_operations = operations;
+            return;
+        }
         self.slide_state.last_chunk_ended_in_list = matches!(self.slide_state.last_element, LastElement::List { .. });
 
         let chunk_operations = mem::take(&mut self.chunk_operations);
@@ -649,9 +667,11 @@ impl<'a> PresentationBuilder<'a> {
             other => panic!("unexpected heading level {other}"),
         };
         if let Some(prefix) = &style.prefix {
-            let mut prefix = prefix.clone();
-            prefix.push(' ');
-            text.0.insert(0, Text::from(prefix));
+            if !prefix.is_empty() {
+                let mut prefix = prefix.clone();
+                prefix.push(' ');
+                text.0.insert(0, Text::from(prefix));
+            }
         }
         text.apply_style(&style.style);
 
@@ -922,11 +942,9 @@ impl<'a> PresentationBuilder<'a> {
             image_registry: &self.image_registry,
             snippet_executor: self.code_executor.clone(),
             theme: &self.theme,
-            presentation_state: &self.presentation_state,
             third_party: self.third_party,
             highlighter: &self.highlighter,
             options: &self.options,
-            slide_number: self.slide_builders.len() + 1,
             font_size: self.slide_font_size(),
         };
         let processor = SnippetProcessor::new(state);
@@ -1162,6 +1180,7 @@ pub enum BuildError {
     InvalidFooter(#[from] InvalidFooterTemplateError),
 }
 
+#[derive(Debug)]
 enum ExecutionMode {
     AlongSnippet,
     ReplaceSnippet,
@@ -1356,9 +1375,10 @@ mod test {
         options: PresentationBuilderOptions,
     ) -> Result<Presentation, BuildError> {
         let theme = raw::PresentationTheme::default();
-        let resources = Resources::new("/tmp", "/tmp", Default::default());
+        let tmp_dir = std::env::temp_dir();
+        let resources = Resources::new(&tmp_dir, &tmp_dir, Default::default());
         let mut third_party = ThirdPartyRender::default();
-        let code_executor = Rc::new(SnippetExecutor::default());
+        let code_executor = Arc::new(SnippetExecutor::default());
         let themes = Themes::default();
         let bindings = KeyBindingsConfig::default();
         let builder = PresentationBuilder::new(
@@ -1444,6 +1464,25 @@ mod test {
         extract_text_lines(&operations)
     }
 
+    #[test]
+    fn empty_heading_prefix() {
+        let frontmatter = r#"
+theme:
+  override:
+    headings:
+      h1:
+        prefix: ""
+"#;
+        let elements = vec![
+            MarkdownElement::FrontMatter(frontmatter.into()),
+            MarkdownElement::Heading { text: "hi".into(), level: 1 },
+        ];
+        let slides = build_presentation(elements).into_slides();
+        let lines = extract_slide_text_lines(slides.into_iter().next().unwrap());
+        let expected_lines = &["hi"];
+        assert_eq!(lines, expected_lines);
+    }
+
     #[test]
     fn prelude_appears_once() {
         let elements = vec![
@@ -1705,6 +1744,18 @@ mod test {
         assert_eq!(slides[0].iter_chunks().count(), 1);
     }
 
+    #[test]
+    fn pause_new_slide() {
+        let elements = vec![
+            MarkdownElement::Paragraph(vec![Line::from("hi")]),
+            MarkdownElement::Comment { comment: "pause".into(), source_position: Default::default() },
+            MarkdownElement::Paragraph(vec![Line::from("bye")]),
+        ];
+        let options = PresentationBuilderOptions { pause_create_new_slide: true, ..Default::default() };
+        let slides = build_presentation_with_options(elements, options).into_slides();
+        assert_eq!(slides.len(), 2);
+    }
+
     #[test]
     fn skip_slide() {
         let elements = vec![
