@@ -4,35 +4,45 @@ use crate::{
         listener::{Command, CommandListener},
         speaker_notes::{SpeakerNotesEvent, SpeakerNotesEventPublisher},
     },
-    config::{KeyBindingsConfig, MaxColumnsAlignment},
+    config::{KeyBindingsConfig, SlideTransitionConfig, SlideTransitionStyleConfig},
     markdown::parse::{MarkdownParser, ParseError},
     presentation::{
-        Presentation,
+        Presentation, Slide,
         builder::{BuildError, PresentationBuilder, PresentationBuilderOptions, Themes},
         diff::PresentationDiffer,
+        poller::{PollableEffect, Poller, PollerCommand},
     },
     render::{
-        ErrorSource, RenderError, RenderResult, TerminalDrawer, TerminalDrawerOptions, operation::RenderAsyncState,
-        properties::WindowSize, validate::OverflowValidator,
+        ErrorSource, RenderError, RenderResult, TerminalDrawer, TerminalDrawerOptions,
+        ascii_scaler::AsciiScaler,
+        engine::{MaxSize, RenderEngine, RenderEngineOptions},
+        operation::{Pollable, RenderAsyncStartPolicy, RenderOperation},
+        properties::WindowSize,
+        validate::OverflowValidator,
     },
     resource::Resources,
     terminal::{
         image::printer::{ImagePrinter, ImageRegistry},
-        printer::TerminalIo,
+        printer::{TerminalCommand, TerminalIo},
+        virt::{ImageBehavior, TerminalGrid, VirtualTerminal},
     },
     theme::{ProcessingThemeError, raw::PresentationTheme},
     third_party::ThirdPartyRender,
+    transitions::{
+        AnimateTransition, AnimationFrame, LinesFrame, TransitionDirection,
+        collapse_horizontal::CollapseHorizontalAnimation, fade::FadeAnimation,
+        slide_horizontal::SlideHorizontalAnimation,
+    },
 };
 use std::{
-    collections::HashSet,
     fmt::Display,
     fs,
     io::{self},
     mem,
     ops::Deref,
     path::Path,
-    rc::Rc,
     sync::Arc,
+    time::{Duration, Instant},
 };
 
 pub struct PresenterOptions {
@@ -41,8 +51,8 @@ pub struct PresenterOptions {
     pub font_size_fallback: u8,
     pub bindings: KeyBindingsConfig,
     pub validate_overflows: bool,
-    pub max_columns: u16,
-    pub max_columns_alignment: MaxColumnsAlignment,
+    pub max_size: MaxSize,
+    pub transition: Option<SlideTransitionConfig>,
 }
 
 /// A slideshow presenter.
@@ -54,13 +64,13 @@ pub struct Presenter<'a> {
     parser: MarkdownParser<'a>,
     resources: Resources,
     third_party: ThirdPartyRender,
-    code_executor: Rc<SnippetExecutor>,
+    code_executor: Arc<SnippetExecutor>,
     state: PresenterState,
-    slides_with_pending_async_renders: HashSet<usize>,
     image_printer: Arc<ImagePrinter>,
     themes: Themes,
     options: PresenterOptions,
     speaker_notes_event_publisher: Option<SpeakerNotesEventPublisher>,
+    poller: Poller,
 }
 
 impl<'a> Presenter<'a> {
@@ -72,7 +82,7 @@ impl<'a> Presenter<'a> {
         parser: MarkdownParser<'a>,
         resources: Resources,
         third_party: ThirdPartyRender,
-        code_executor: Rc<SnippetExecutor>,
+        code_executor: Arc<SnippetExecutor>,
         themes: Themes,
         image_printer: Arc<ImagePrinter>,
         options: PresenterOptions,
@@ -86,11 +96,11 @@ impl<'a> Presenter<'a> {
             third_party,
             code_executor,
             state: PresenterState::Empty,
-            slides_with_pending_async_renders: HashSet::new(),
             image_printer,
             themes,
             options,
             speaker_notes_event_publisher,
+            poller: Poller::launch(),
         }
     }
 
@@ -100,21 +110,19 @@ impl<'a> Presenter<'a> {
             self.resources.watch_presentation_file(path.to_path_buf());
         }
         self.state = PresenterState::Presenting(Presentation::from(vec![]));
-        self.try_reload(path, true);
+        self.try_reload(path, true)?;
 
         let drawer_options = TerminalDrawerOptions {
             font_size_fallback: self.options.font_size_fallback,
-            max_columns: self.options.max_columns,
-            max_columns_alignment: self.options.max_columns_alignment,
+            max_size: self.options.max_size.clone(),
         };
         let mut drawer = TerminalDrawer::new(self.image_printer.clone(), drawer_options)?;
         loop {
             // Poll async renders once before we draw just in case.
-            self.poll_async_renders()?;
             self.render(&mut drawer)?;
 
             loop {
-                if self.poll_async_renders()? {
+                if self.process_poller_effects()? {
                     self.render(&mut drawer)?;
                 }
 
@@ -140,10 +148,19 @@ impl<'a> Presenter<'a> {
                         break;
                     }
                     CommandSideEffect::Reload => {
-                        self.try_reload(path, false);
+                        self.try_reload(path, false)?;
                         break;
                     }
                     CommandSideEffect::Redraw => {
+                        self.try_scale_transition_images()?;
+                        break;
+                    }
+                    CommandSideEffect::AnimateNextSlide => {
+                        self.animate_next_slide(&mut drawer)?;
+                        break;
+                    }
+                    CommandSideEffect::AnimatePreviousSlide => {
+                        self.animate_previous_slide(&mut drawer)?;
                         break;
                     }
                     CommandSideEffect::None => (),
@@ -155,6 +172,36 @@ impl<'a> Presenter<'a> {
         }
     }
 
+    fn process_poller_effects(&mut self) -> Result<bool, PresentationError> {
+        let current_slide = match &self.state {
+            PresenterState::Presenting(presentation)
+            | PresenterState::SlideIndex(presentation)
+            | PresenterState::KeyBindings(presentation)
+            | PresenterState::Failure { presentation, .. } => presentation.current_slide_index(),
+            PresenterState::Empty => usize::MAX,
+        };
+        let mut refreshed = false;
+        let mut needs_render = false;
+        while let Some(effect) = self.poller.next_effect() {
+            match effect {
+                PollableEffect::RefreshSlide(index) => {
+                    needs_render = needs_render || index == current_slide;
+                    refreshed = true;
+                }
+                PollableEffect::DisplayError { slide, error } => {
+                    let presentation = mem::take(&mut self.state).into_presentation();
+                    self.state =
+                        PresenterState::failure(error, presentation, ErrorSource::Slide(slide + 1), FailureMode::Other);
+                    needs_render = true;
+                }
+            }
+        }
+        if refreshed {
+            self.try_scale_transition_images()?;
+        }
+        Ok(needs_render)
+    }
+
     fn publish_event(&self, event: SpeakerNotesEvent) -> io::Result<()> {
         if let Some(publisher) = &self.speaker_notes_event_publisher {
             publisher.send(event)?;
@@ -180,27 +227,6 @@ impl<'a> Presenter<'a> {
         }
     }
 
-    fn poll_async_renders(&mut self) -> Result<bool, RenderError> {
-        if matches!(self.state, PresenterState::Failure { .. }) {
-            return Ok(false);
-        }
-        let current_index = self.state.presentation().current_slide_index();
-        if self.slides_with_pending_async_renders.contains(&current_index) {
-            let state = self.state.presentation_mut().poll_slide_async_renders();
-            match state {
-                RenderAsyncState::NotStarted | RenderAsyncState::Rendering { modified: false } => (),
-                RenderAsyncState::Rendering { modified: true } => {
-                    return Ok(true);
-                }
-                RenderAsyncState::Rendered | RenderAsyncState::JustFinishedRendering => {
-                    self.slides_with_pending_async_renders.remove(&current_index);
-                    return Ok(true);
-                }
-            };
-        }
-        Ok(false)
-    }
-
     fn render(&mut self, drawer: &mut TerminalDrawer) -> RenderResult {
         let result = match &self.state {
             PresenterState::Presenting(presentation) => {
@@ -256,16 +282,37 @@ impl<'a> Presenter<'a> {
             }
         };
         let needs_redraw = match command {
-            Command::Next => presentation.jump_next(),
+            Command::Next => {
+                let current_slide = presentation.current_slide_index();
+                if !presentation.jump_next() {
+                    false
+                } else if presentation.current_slide_index() != current_slide {
+                    return CommandSideEffect::AnimateNextSlide;
+                } else {
+                    true
+                }
+            }
             Command::NextFast => presentation.jump_next_fast(),
-            Command::Previous => presentation.jump_previous(),
+            Command::Previous => {
+                let current_slide = presentation.current_slide_index();
+                if !presentation.jump_previous() {
+                    false
+                } else if presentation.current_slide_index() != current_slide {
+                    return CommandSideEffect::AnimatePreviousSlide;
+                } else {
+                    true
+                }
+            }
             Command::PreviousFast => presentation.jump_previous_fast(),
             Command::FirstSlide => presentation.jump_first_slide(),
             Command::LastSlide => presentation.jump_last_slide(),
             Command::GoToSlide(number) => presentation.go_to_slide(number.saturating_sub(1) as usize),
             Command::RenderAsyncOperations => {
-                if presentation.trigger_slide_async_renders() {
-                    self.slides_with_pending_async_renders.insert(self.state.presentation().current_slide_index());
+                let pollables = Self::trigger_slide_async_renders(presentation);
+                if !pollables.is_empty() {
+                    for pollable in pollables {
+                        self.poller.send(PollerCommand::Poll { pollable, slide: presentation.current_slide_index() });
+                    }
                     return CommandSideEffect::Redraw;
                 } else {
                     return CommandSideEffect::None;
@@ -292,11 +339,11 @@ impl<'a> Presenter<'a> {
         if needs_redraw { CommandSideEffect::Redraw } else { CommandSideEffect::None }
     }
 
-    fn try_reload(&mut self, path: &Path, force: bool) {
+    fn try_reload(&mut self, path: &Path, force: bool) -> RenderResult {
         if matches!(self.options.mode, PresentMode::Presentation) && !force {
-            return;
+            return Ok(());
         }
-        self.slides_with_pending_async_renders.clear();
+        self.poller.send(PollerCommand::Reset);
         self.resources.clear_watches();
         match self.load_presentation(path) {
             Ok(mut presentation) => {
@@ -308,14 +355,40 @@ impl<'a> Presenter<'a> {
                     presentation.go_to_slide(current.current_slide_index());
                     presentation.jump_chunk(current.current_chunk());
                 }
-                self.slides_with_pending_async_renders = presentation.slides_with_async_renders().into_iter().collect();
+                self.start_automatic_async_renders(&mut presentation);
                 self.state = self.validate_overflows(presentation);
+                self.try_scale_transition_images()?;
             }
             Err(e) => {
                 let presentation = mem::take(&mut self.state).into_presentation();
                 self.state = PresenterState::failure(e, presentation, ErrorSource::Presentation, FailureMode::Other);
             }
         };
+        Ok(())
+    }
+
+    fn try_scale_transition_images(&self) -> RenderResult {
+        if self.options.transition.is_none() {
+            return Ok(());
+        }
+        let options = RenderEngineOptions { max_size: self.options.max_size.clone(), ..Default::default() };
+        let scaler = AsciiScaler::new(options);
+        let dimensions = WindowSize::current(self.options.font_size_fallback)?;
+        scaler.process(self.state.presentation(), &dimensions)?;
+        Ok(())
+    }
+
+    fn trigger_slide_async_renders(presentation: &mut Presentation) -> Vec<Box<dyn Pollable>> {
+        let slide = presentation.current_slide_mut();
+        let mut pollables = Vec::new();
+        for operation in slide.iter_visible_operations_mut() {
+            if let RenderOperation::RenderAsync(operation) = operation {
+                if let RenderAsyncStartPolicy::OnDemand = operation.start_policy() {
+                    pollables.push(operation.pollable());
+                }
+            }
+        }
+        pollables
     }
 
     fn is_displaying_other_error(&self) -> bool {
@@ -348,7 +421,7 @@ impl<'a> Presenter<'a> {
             &mut self.third_party,
             self.code_executor.clone(),
             &self.themes,
-            ImageRegistry(self.image_printer.clone()),
+            ImageRegistry::new(self.image_printer.clone()),
             self.options.bindings.clone(),
             self.options.builder_options.clone(),
         )?
@@ -386,6 +459,140 @@ impl<'a> Presenter<'a> {
             drawer.terminal.resume();
         }
     }
+
+    fn animate_next_slide(&mut self, drawer: &mut TerminalDrawer) -> RenderResult {
+        let Some(config) = self.options.transition.clone() else {
+            return Ok(());
+        };
+
+        let options = drawer.render_engine_options();
+        let presentation = self.state.presentation_mut();
+        let dimensions = WindowSize::current(self.options.font_size_fallback)?;
+        presentation.jump_previous();
+        let left = Self::virtual_render(presentation.current_slide(), dimensions.clone(), &options)?;
+        presentation.jump_next();
+        let right = Self::virtual_render(presentation.current_slide(), dimensions.clone(), &options)?;
+        let direction = TransitionDirection::Next;
+        self.animate_transition(drawer, left, right, direction, dimensions, config)
+    }
+
+    fn animate_previous_slide(&mut self, drawer: &mut TerminalDrawer) -> RenderResult {
+        let Some(config) = self.options.transition.clone() else {
+            return Ok(());
+        };
+
+        let options = drawer.render_engine_options();
+        let presentation = self.state.presentation_mut();
+        let dimensions = WindowSize::current(self.options.font_size_fallback)?;
+        presentation.jump_next();
+
+        // Re-borrow to avoid calling fns above while mutably borrowing
+        let presentation = self.state.presentation_mut();
+
+        let right = Self::virtual_render(presentation.current_slide(), dimensions.clone(), &options)?;
+        presentation.jump_previous();
+        let left = Self::virtual_render(presentation.current_slide(), dimensions.clone(), &options)?;
+        let direction = TransitionDirection::Previous;
+        self.animate_transition(drawer, left, right, direction, dimensions, config)
+    }
+
+    fn animate_transition(
+        &mut self,
+        drawer: &mut TerminalDrawer,
+        left: TerminalGrid,
+        right: TerminalGrid,
+        direction: TransitionDirection,
+        dimensions: WindowSize,
+        config: SlideTransitionConfig,
+    ) -> RenderResult {
+        let first = match &direction {
+            TransitionDirection::Next => left.clone(),
+            TransitionDirection::Previous => right.clone(),
+        };
+        match &config.animation {
+            SlideTransitionStyleConfig::SlideHorizontal => self.run_animation(
+                drawer,
+                first,
+                SlideHorizontalAnimation::new(left, right, dimensions, direction),
+                config,
+            ),
+            SlideTransitionStyleConfig::Fade => {
+                self.run_animation(drawer, first, FadeAnimation::new(left, right, direction), config)
+            }
+            SlideTransitionStyleConfig::CollapseHorizontal => {
+                self.run_animation(drawer, first, CollapseHorizontalAnimation::new(left, right, direction), config)
+            }
+        }
+    }
+
+    fn run_animation<T>(
+        &mut self,
+        drawer: &mut TerminalDrawer,
+        first: TerminalGrid,
+        animation: T,
+        config: SlideTransitionConfig,
+    ) -> RenderResult
+    where
+        T: AnimateTransition,
+    {
+        let total_time = Duration::from_millis(config.duration_millis as u64);
+        let frames: usize = config.frames;
+        let total_frames = animation.total_frames();
+        let step = total_time / (frames as u32 * 2);
+        let mut last_frame_index = 0;
+        let mut frame_index = 1;
+        // Render the first frame as text to have images as ascii
+        Self::render_frame(&LinesFrame::from(&first).build_commands(), drawer)?;
+        while frame_index < total_frames {
+            let start = Instant::now();
+            let frame = animation.build_frame(frame_index, last_frame_index);
+            let commands = frame.build_commands();
+            Self::render_frame(&commands, drawer)?;
+
+            let elapsed = start.elapsed();
+            let sleep_needed = step.saturating_sub(elapsed);
+            if sleep_needed.as_millis() > 0 {
+                std::thread::sleep(step);
+            }
+            last_frame_index = frame_index;
+            frame_index += total_frames.div_ceil(frames);
+        }
+        Ok(())
+    }
+
+    fn render_frame(commands: &[TerminalCommand<'_>], drawer: &mut TerminalDrawer) -> RenderResult {
+        drawer.terminal.execute(&TerminalCommand::BeginUpdate)?;
+        for command in commands {
+            drawer.terminal.execute(command)?;
+        }
+        drawer.terminal.execute(&TerminalCommand::EndUpdate)?;
+        drawer.terminal.execute(&TerminalCommand::Flush)?;
+        Ok(())
+    }
+
+    fn virtual_render(
+        slide: &Slide,
+        dimensions: WindowSize,
+        options: &RenderEngineOptions,
+    ) -> Result<TerminalGrid, RenderError> {
+        let mut term = VirtualTerminal::new(dimensions.clone(), ImageBehavior::PrintAscii);
+        let engine = RenderEngine::new(&mut term, dimensions.clone(), options.clone());
+        engine.render(slide.iter_visible_operations())?;
+        Ok(term.into_contents())
+    }
+
+    fn start_automatic_async_renders(&self, presentation: &mut Presentation) {
+        for (index, slide) in presentation.iter_slides_mut().enumerate() {
+            for operation in slide.iter_operations_mut() {
+                if let RenderOperation::RenderAsync(operation) = operation {
+                    if let RenderAsyncStartPolicy::Automatic = operation.start_policy() {
+                        let pollable = operation.pollable();
+                        self.poller.send(PollerCommand::Poll { pollable, slide: index });
+                    }
+                }
+            }
+        }
+    }
 }
 
 enum CommandSideEffect {
@@ -393,6 +600,8 @@ enum CommandSideEffect {
     Suspend,
     Redraw,
     Reload,
+    AnimateNextSlide,
+    AnimatePreviousSlide,
     None,
 }
 
