@@ -5,7 +5,7 @@ use crate::{
         text_style::{Color, Colors, TextStyle},
     },
     render::{RenderError, RenderResult, layout::Positioning},
-    terminal::printer::{TerminalIo, TextProperties},
+    terminal::printer::{TerminalCommand, TerminalIo},
 };
 
 /// Draws text on the screen.
@@ -21,7 +21,7 @@ pub(crate) struct TextDrawer<'a> {
     draw_block: bool,
     block_color: Option<Color>,
     repeat_prefix: bool,
-    properties: TextProperties,
+    center_newlines: bool,
 }
 
 impl<'a> TextDrawer<'a> {
@@ -56,7 +56,7 @@ impl<'a> TextDrawer<'a> {
                 draw_block: false,
                 block_color: None,
                 repeat_prefix: false,
-                properties: TextProperties { height: line.font_size() },
+                center_newlines: false,
             })
         }
     }
@@ -72,6 +72,11 @@ impl<'a> TextDrawer<'a> {
         self
     }
 
+    pub(crate) fn center_newlines(mut self, value: bool) -> Self {
+        self.center_newlines = value;
+        self
+    }
+
     /// Draw text on the given handle.
     ///
     /// This performs word splitting and word wrapping.
@@ -80,33 +85,42 @@ impl<'a> TextDrawer<'a> {
         T: TerminalIo,
     {
         let mut line_length: u16 = 0;
-        terminal.move_to_column(self.positioning.start_column)?;
+        terminal.execute(&TerminalCommand::MoveToColumn(self.positioning.start_column))?;
+        let font_size = self.line.font_size();
 
         // Print the prefix at the beginning of the line.
         if self.prefix_width > 0 {
             let Text { content, style } = self.prefix.text();
-            terminal.print_text(content, style, &self.properties)?;
+            terminal.execute(&TerminalCommand::PrintText { content, style: *style })?;
         }
         for (line_index, line) in self.line.split(self.positioning.max_line_length as usize).enumerate() {
             if line_index > 0 {
                 // Complete the current line's block to the right before moving down.
                 self.print_block_background(line_length, terminal)?;
-                terminal.move_down(self.properties.height as u16)?;
-                terminal.move_to_column(self.positioning.start_column)?;
+                terminal.execute(&TerminalCommand::MoveDown(font_size as u16))?;
+                let start_column = match self.center_newlines {
+                    true => {
+                        let line_width = line.iter().map(|l| l.width()).sum::<usize>() as u16;
+                        let extra_space = self.positioning.max_line_length.saturating_sub(line_width);
+                        self.positioning.start_column + extra_space / 2
+                    }
+                    false => self.positioning.start_column,
+                };
+                terminal.execute(&TerminalCommand::MoveToColumn(start_column))?;
                 line_length = 0;
 
                 // Complete the new line in this block to the left where the prefix would be.
                 if self.prefix_width > 0 {
                     if self.repeat_prefix {
                         let Text { content, style } = self.prefix.text();
-                        terminal.print_text(content, style, &self.properties)?;
+                        terminal.execute(&TerminalCommand::PrintText { content, style: *style })?;
                     } else {
                         if let Some(color) = self.block_color {
-                            terminal.set_background_color(color)?;
+                            terminal.execute(&TerminalCommand::SetBackgroundColor(color))?;
                         }
-                        let text = " ".repeat(self.prefix_width as usize / self.properties.height as usize);
-                        let style = TextStyle::default().size(self.properties.height);
-                        terminal.print_text(&text, &style, &self.properties)?;
+                        let text = " ".repeat(self.prefix_width as usize / font_size as usize);
+                        let style = TextStyle::default().size(font_size);
+                        terminal.execute(&TerminalCommand::PrintText { content: &text, style })?;
                     }
                 }
             }
@@ -114,12 +128,12 @@ impl<'a> TextDrawer<'a> {
                 line_length = line_length.saturating_add(chunk.width() as u16);
 
                 let (text, style) = chunk.into_parts();
-                terminal.print_text(text, &style, &self.properties)?;
+                terminal.execute(&TerminalCommand::PrintText { content: text, style })?;
 
                 // Crossterm resets colors if any attributes are set so let's just re-apply colors
                 // if the format has anything on it at all.
                 if style != Default::default() {
-                    terminal.set_colors(*self.default_colors)?;
+                    terminal.execute(&TerminalCommand::SetColors(*self.default_colors))?;
                 }
             }
         }
@@ -135,12 +149,13 @@ impl<'a> TextDrawer<'a> {
             let remaining =
                 self.positioning.max_line_length.saturating_sub(line_length).saturating_add(self.right_padding_length);
             if remaining > 0 {
+                let font_size = self.line.font_size();
                 if let Some(color) = self.block_color {
-                    terminal.set_background_color(color)?;
+                    terminal.execute(&TerminalCommand::SetBackgroundColor(color))?;
                 }
-                let text = " ".repeat(remaining as usize / self.properties.height as usize);
-                let style = TextStyle::default().size(self.properties.height);
-                terminal.print_text(&text, &style, &self.properties)?;
+                let text = " ".repeat(remaining as usize / font_size as usize);
+                let style = TextStyle::default().size(font_size);
+                terminal.execute(&TerminalCommand::PrintText { content: &text, style })?;
             }
         }
         Ok(())
@@ -150,7 +165,7 @@ impl<'a> TextDrawer<'a> {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::terminal::image::{Image, printer::PrintOptions};
+    use crate::terminal::printer::TerminalError;
     use std::io;
     use unicode_width::UnicodeWidthStr;
 
@@ -172,28 +187,6 @@ mod tests {
             self.instructions.push(instruction);
             Ok(())
         }
-    }
-
-    impl TerminalIo for TerminalBuf {
-        fn begin_update(&mut self) -> std::io::Result<()> {
-            unimplemented!()
-        }
-
-        fn end_update(&mut self) -> std::io::Result<()> {
-            unimplemented!()
-        }
-
-        fn cursor_row(&self) -> u16 {
-            self.cursor_row
-        }
-
-        fn move_to(&mut self, _column: u16, _row: u16) -> std::io::Result<()> {
-            unimplemented!()
-        }
-
-        fn move_to_row(&mut self, _row: u16) -> std::io::Result<()> {
-            unimplemented!()
-        }
 
         fn move_to_column(&mut self, column: u16) -> std::io::Result<()> {
             self.push(Instruction::MoveToColumn(column))
@@ -203,11 +196,7 @@ mod tests {
             self.push(Instruction::MoveDown(amount))
         }
 
-        fn move_to_next_line(&mut self) -> std::io::Result<()> {
-            unimplemented!()
-        }
-
-        fn print_text(&mut self, content: &str, style: &TextStyle, _properties: &TextProperties) -> io::Result<()> {
+        fn print_text(&mut self, content: &str, style: &TextStyle) -> io::Result<()> {
             let content = content.to_string();
             if content.is_empty() {
                 return Ok(());
@@ -232,21 +221,35 @@ mod tests {
         fn flush(&mut self) -> std::io::Result<()> {
             Ok(())
         }
+    }
 
-        fn print_image(
-            &mut self,
-            _image: &Image,
-            _options: &PrintOptions,
-        ) -> Result<(), crate::terminal::image::printer::PrintImageError> {
-            unimplemented!()
+    impl TerminalIo for TerminalBuf {
+        fn execute(&mut self, command: &TerminalCommand<'_>) -> Result<(), TerminalError> {
+            use TerminalCommand::*;
+            match command {
+                BeginUpdate
+                | EndUpdate
+                | MoveToRow(_)
+                | MoveToNextLine
+                | MoveTo { .. }
+                | MoveRight(_)
+                | MoveLeft(_)
+                | PrintImage { .. } => {
+                    unimplemented!()
+                }
+                MoveToColumn(column) => self.move_to_column(*column)?,
+                MoveDown(amount) => self.move_down(*amount)?,
+                PrintText { content, style } => self.print_text(content, style)?,
+                ClearScreen => self.clear_screen()?,
+                SetColors(colors) => self.set_colors(*colors)?,
+                SetBackgroundColor(color) => self.set_background_color(*color)?,
+                Flush => self.flush()?,
+            };
+            Ok(())
         }
 
-        fn suspend(&mut self) {
-            unimplemented!()
-        }
-
-        fn resume(&mut self) {
-            unimplemented!()
+        fn cursor_row(&self) -> u16 {
+            self.cursor_row
         }
     }
 
@@ -255,6 +258,7 @@ mod tests {
         positioning: Positioning,
         right_padding_length: u16,
         repeat_prefix_on_wrap: bool,
+        center_newlines: bool,
     }
 
     impl TestDrawer {
@@ -278,12 +282,18 @@ mod tests {
             self
         }
 
+        fn center_newlines(mut self) -> Self {
+            self.center_newlines = true;
+            self
+        }
+
         fn draw<L: Into<WeightedLine>>(self, line: L) -> Vec<Instruction> {
             let line = line.into();
             let colors = Default::default();
             let drawer = TextDrawer::new(&self.prefix, self.right_padding_length, &line, self.positioning, &colors, 0)
                 .expect("failed to create drawer")
-                .repeat_prefix_on_wrap(self.repeat_prefix_on_wrap);
+                .repeat_prefix_on_wrap(self.repeat_prefix_on_wrap)
+                .center_newlines(self.center_newlines);
             let mut buf = TerminalBuf::default();
             drawer.draw(&mut buf).expect("drawing failed");
             buf.instructions
@@ -297,6 +307,7 @@ mod tests {
                 positioning: Positioning { max_line_length: 100, start_column: 0 },
                 right_padding_length: 0,
                 repeat_prefix_on_wrap: false,
+                center_newlines: false,
             }
         }
     }
@@ -350,4 +361,18 @@ mod tests {
         ];
         assert_eq!(instructions, expected);
     }
+
+    #[test]
+    fn center_newlines() {
+        let text = WeightedLine::from(vec![Text::from("hello world foo")]);
+        let instructions = TestDrawer::default().center_newlines().max_line_length(11).draw(text);
+        let expected = &[
+            Instruction::MoveToColumn(0),
+            Instruction::PrintText { content: "hello world".into(), font_size: 1 },
+            Instruction::MoveDown(1),
+            Instruction::MoveToColumn(4),
+            Instruction::PrintText { content: "foo".into(), font_size: 1 },
+        ];
+        assert_eq!(instructions, expected);
+    }
 }
