@@ -8,40 +8,39 @@ use super::{
 };
 use crate::{
     markdown::text_style::{Color, PaletteColorError},
-    render::properties::CursorPosition,
-    terminal::GraphicsMode,
+    terminal::{
+        GraphicsMode,
+        printer::{TerminalError, TerminalIo},
+    },
 };
 use image::{DynamicImage, ImageError};
 use std::{
     borrow::Cow,
+    collections::HashMap,
     fmt, io,
-    path::{Path, PathBuf},
-    sync::Arc,
+    path::PathBuf,
+    sync::{Arc, Mutex},
 };
 
 pub(crate) trait PrintImage {
     type Image: ImageProperties;
 
     /// Register an image.
-    fn register(&self, image: DynamicImage) -> Result<Self::Image, RegisterImageError>;
+    fn register(&self, spec: ImageSpec) -> Result<Self::Image, RegisterImageError>;
 
-    /// Load and register an image from the given path.
-    fn register_from_path<P: AsRef<Path>>(&self, path: P) -> Result<Self::Image, RegisterImageError>;
-
-    fn print<W>(&self, image: &Self::Image, options: &PrintOptions, writer: &mut W) -> Result<(), PrintImageError>
+    fn print<T>(&self, image: &Self::Image, options: &PrintOptions, terminal: &mut T) -> Result<(), PrintImageError>
     where
-        W: io::Write;
+        T: TerminalIo;
 }
 
 pub(crate) trait ImageProperties {
     fn dimensions(&self) -> (u32, u32);
 }
 
-#[derive(Debug)]
+#[derive(Clone, Debug, PartialEq)]
 pub(crate) struct PrintOptions {
     pub(crate) columns: u16,
     pub(crate) rows: u16,
-    pub(crate) cursor_position: CursorPosition,
     pub(crate) z_index: i32,
     pub(crate) background_color: Option<Color>,
     // Width/height in pixels.
@@ -119,52 +118,49 @@ impl ImagePrinter {
 impl PrintImage for ImagePrinter {
     type Image = TerminalImage;
 
-    fn register(&self, image: DynamicImage) -> Result<Self::Image, RegisterImageError> {
+    fn register(&self, spec: ImageSpec) -> Result<Self::Image, RegisterImageError> {
         let image = match self {
-            Self::Kitty(printer) => TerminalImage::Kitty(printer.register(image)?),
-            Self::Iterm(printer) => TerminalImage::Iterm(printer.register(image)?),
-            Self::Ascii(printer) => TerminalImage::Ascii(printer.register(image)?),
+            Self::Kitty(printer) => TerminalImage::Kitty(printer.register(spec)?),
+            Self::Iterm(printer) => TerminalImage::Iterm(printer.register(spec)?),
+            Self::Ascii(printer) => TerminalImage::Ascii(printer.register(spec)?),
             Self::Null => return Err(RegisterImageError::Unsupported),
             #[cfg(feature = "sixel")]
-            Self::Sixel(printer) => TerminalImage::Sixel(printer.register(image)?),
+            Self::Sixel(printer) => TerminalImage::Sixel(printer.register(spec)?),
         };
         Ok(image)
     }
 
-    fn register_from_path<P: AsRef<Path>>(&self, path: P) -> Result<Self::Image, RegisterImageError> {
-        let image = match self {
-            Self::Kitty(printer) => TerminalImage::Kitty(printer.register_from_path(path)?),
-            Self::Iterm(printer) => TerminalImage::Iterm(printer.register_from_path(path)?),
-            Self::Ascii(printer) => TerminalImage::Ascii(printer.register_from_path(path)?),
-            Self::Null => return Err(RegisterImageError::Unsupported),
-            #[cfg(feature = "sixel")]
-            Self::Sixel(printer) => TerminalImage::Sixel(printer.register_from_path(path)?),
-        };
-        Ok(image)
-    }
-
-    fn print<W>(&self, image: &Self::Image, options: &PrintOptions, writer: &mut W) -> Result<(), PrintImageError>
+    fn print<T>(&self, image: &Self::Image, options: &PrintOptions, terminal: &mut T) -> Result<(), PrintImageError>
     where
-        W: io::Write,
+        T: TerminalIo,
     {
         match (self, image) {
-            (Self::Kitty(printer), TerminalImage::Kitty(image)) => printer.print(image, options, writer),
-            (Self::Iterm(printer), TerminalImage::Iterm(image)) => printer.print(image, options, writer),
-            (Self::Ascii(printer), TerminalImage::Ascii(image)) => printer.print(image, options, writer),
+            (Self::Kitty(printer), TerminalImage::Kitty(image)) => printer.print(image, options, terminal),
+            (Self::Iterm(printer), TerminalImage::Iterm(image)) => printer.print(image, options, terminal),
+            (Self::Ascii(printer), TerminalImage::Ascii(image)) => printer.print(image, options, terminal),
             (Self::Null, _) => Ok(()),
             #[cfg(feature = "sixel")]
-            (Self::Sixel(printer), TerminalImage::Sixel(image)) => printer.print(image, options, writer),
+            (Self::Sixel(printer), TerminalImage::Sixel(image)) => printer.print(image, options, terminal),
             _ => Err(PrintImageError::Unsupported),
         }
     }
 }
 
 #[derive(Clone, Default)]
-pub(crate) struct ImageRegistry(pub Arc<ImagePrinter>);
+pub(crate) struct ImageRegistry {
+    printer: Arc<ImagePrinter>,
+    images: Arc<Mutex<HashMap<PathBuf, Image>>>,
+}
+
+impl ImageRegistry {
+    pub fn new(printer: Arc<ImagePrinter>) -> Self {
+        Self { printer, images: Default::default() }
+    }
+}
 
 impl fmt::Debug for ImageRegistry {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        let inner = match self.0.as_ref() {
+        let inner = match self.printer.as_ref() {
             ImagePrinter::Kitty(_) => "Kitty",
             ImagePrinter::Iterm(_) => "Iterm",
             ImagePrinter::Ascii(_) => "Ascii",
@@ -177,19 +173,36 @@ impl fmt::Debug for ImageRegistry {
 }
 
 impl ImageRegistry {
-    pub(crate) fn register_image(&self, image: DynamicImage) -> Result<Image, RegisterImageError> {
-        let resource = self.0.register(image)?;
-        let image = Image::new(resource, ImageSource::Generated);
+    pub(crate) fn register(&self, spec: ImageSpec) -> Result<Image, RegisterImageError> {
+        let mut images = self.images.lock().unwrap();
+        let (source, cache_key) = match &spec {
+            ImageSpec::Generated(_) => (ImageSource::Generated, None),
+            ImageSpec::Filesystem(path) => {
+                // Return if already cached
+                if let Some(image) = images.get(path) {
+                    return Ok(image.clone());
+                }
+                (ImageSource::Filesystem(path.clone()), Some(path.clone()))
+            }
+        };
+        let resource = self.printer.register(spec)?;
+        let image = Image::new(resource, source);
+        if let Some(key) = cache_key {
+            images.insert(key.clone(), image.clone());
+        }
         Ok(image)
     }
 
-    pub(crate) fn register_resource(&self, path: PathBuf) -> Result<Image, RegisterImageError> {
-        let resource = self.0.register_from_path(&path)?;
-        let image = Image::new(resource, ImageSource::Filesystem(path));
-        Ok(image)
+    pub(crate) fn clear(&self) {
+        self.images.lock().unwrap().clear();
     }
 }
 
+pub(crate) enum ImageSpec {
+    Generated(DynamicImage),
+    Filesystem(PathBuf),
+}
+
 #[derive(Debug, thiserror::Error)]
 pub(crate) enum CreatePrinterError {
     #[error("io: {0}")]
@@ -217,6 +230,15 @@ impl From<PaletteColorError> for PrintImageError {
     }
 }
 
+impl From<TerminalError> for PrintImageError {
+    fn from(e: TerminalError) -> Self {
+        match e {
+            TerminalError::Io(e) => Self::Io(e),
+            TerminalError::Image(e) => e,
+        }
+    }
+}
+
 #[derive(Debug, thiserror::Error)]
 pub(crate) enum RegisterImageError {
     #[error(transparent)]
