@@ -14,23 +14,37 @@ use std::{
     sync::Arc,
 };
 
+#[derive(Debug, PartialEq)]
+pub(crate) enum TerminalCommand<'a> {
+    BeginUpdate,
+    EndUpdate,
+    MoveTo { column: u16, row: u16 },
+    MoveToRow(u16),
+    MoveToColumn(u16),
+    MoveDown(u16),
+    MoveRight(u16),
+    MoveLeft(u16),
+    MoveToNextLine,
+    PrintText { content: &'a str, style: TextStyle },
+    ClearScreen,
+    SetColors(Colors),
+    SetBackgroundColor(Color),
+    Flush,
+    PrintImage { image: Image, options: PrintOptions },
+}
+
 pub(crate) trait TerminalIo {
-    fn begin_update(&mut self) -> io::Result<()>;
-    fn end_update(&mut self) -> io::Result<()>;
+    fn execute(&mut self, command: &TerminalCommand<'_>) -> Result<(), TerminalError>;
     fn cursor_row(&self) -> u16;
-    fn move_to(&mut self, column: u16, row: u16) -> io::Result<()>;
-    fn move_to_row(&mut self, row: u16) -> io::Result<()>;
-    fn move_to_column(&mut self, column: u16) -> io::Result<()>;
-    fn move_down(&mut self, amount: u16) -> io::Result<()>;
-    fn move_to_next_line(&mut self) -> io::Result<()>;
-    fn print_text(&mut self, content: &str, style: &TextStyle, properties: &TextProperties) -> io::Result<()>;
-    fn clear_screen(&mut self) -> io::Result<()>;
-    fn set_colors(&mut self, colors: Colors) -> io::Result<()>;
-    fn set_background_color(&mut self, color: Color) -> io::Result<()>;
-    fn flush(&mut self) -> io::Result<()>;
-    fn print_image(&mut self, image: &Image, options: &PrintOptions) -> Result<(), PrintImageError>;
-    fn suspend(&mut self);
-    fn resume(&mut self);
+}
+
+#[derive(Debug, thiserror::Error)]
+pub(crate) enum TerminalError {
+    #[error("io: {0}")]
+    Io(#[from] io::Error),
+
+    #[error("image: {0}")]
+    Image(#[from] PrintImageError),
 }
 
 /// A wrapper over the terminal write handle.
@@ -46,9 +60,7 @@ impl<I: TerminalWrite> Terminal<I> {
         writer.init()?;
         Ok(Self { writer, image_printer, cursor_row: 0, current_row_height: 1 })
     }
-}
 
-impl<I: TerminalWrite> TerminalIo for Terminal<I> {
     fn begin_update(&mut self) -> io::Result<()> {
         self.writer.queue(terminal::BeginSynchronizedUpdate)?;
         Ok(())
@@ -59,10 +71,6 @@ impl<I: TerminalWrite> TerminalIo for Terminal<I> {
         Ok(())
     }
 
-    fn cursor_row(&self) -> u16 {
-        self.cursor_row
-    }
-
     fn move_to(&mut self, column: u16, row: u16) -> io::Result<()> {
         self.writer.queue(cursor::MoveTo(column, row))?;
         self.cursor_row = row;
@@ -86,6 +94,16 @@ impl<I: TerminalWrite> TerminalIo for Terminal<I> {
         Ok(())
     }
 
+    fn move_right(&mut self, amount: u16) -> io::Result<()> {
+        self.writer.queue(cursor::MoveRight(amount))?;
+        Ok(())
+    }
+
+    fn move_left(&mut self, amount: u16) -> io::Result<()> {
+        self.writer.queue(cursor::MoveLeft(amount))?;
+        Ok(())
+    }
+
     fn move_to_next_line(&mut self) -> io::Result<()> {
         let amount = self.current_row_height;
         self.writer.queue(cursor::MoveToNextLine(amount))?;
@@ -94,16 +112,17 @@ impl<I: TerminalWrite> TerminalIo for Terminal<I> {
         Ok(())
     }
 
-    fn print_text(&mut self, content: &str, style: &TextStyle, properties: &TextProperties) -> io::Result<()> {
+    fn print_text(&mut self, content: &str, style: &TextStyle) -> io::Result<()> {
         let content = style.apply(content);
         self.writer.queue(style::PrintStyledContent(content))?;
-        self.current_row_height = self.current_row_height.max(properties.height as u16);
+        self.current_row_height = self.current_row_height.max(style.size as u16);
         Ok(())
     }
 
     fn clear_screen(&mut self) -> io::Result<()> {
         self.writer.queue(terminal::Clear(terminal::ClearType::All))?;
         self.cursor_row = 0;
+        self.current_row_height = 1;
         Ok(())
     }
 
@@ -126,38 +145,55 @@ impl<I: TerminalWrite> TerminalIo for Terminal<I> {
     }
 
     fn print_image(&mut self, image: &Image, options: &PrintOptions) -> Result<(), PrintImageError> {
-        self.move_to_column(options.cursor_position.column)?;
-        self.image_printer.print(&image.image, options, &mut self.writer)?;
+        let image_printer = self.image_printer.clone();
+        image_printer.print(image.image(), options, self)?;
         self.cursor_row += options.rows;
         Ok(())
     }
 
-    fn suspend(&mut self) {
+    pub(crate) fn suspend(&mut self) {
         self.writer.deinit();
     }
 
-    fn resume(&mut self) {
+    pub(crate) fn resume(&mut self) {
         let _ = self.writer.init();
     }
 }
 
+impl<I: TerminalWrite> TerminalIo for Terminal<I> {
+    fn execute(&mut self, command: &TerminalCommand<'_>) -> Result<(), TerminalError> {
+        use TerminalCommand::*;
+        match command {
+            BeginUpdate => self.begin_update()?,
+            EndUpdate => self.end_update()?,
+            MoveTo { column, row } => self.move_to(*column, *row)?,
+            MoveToRow(row) => self.move_to_row(*row)?,
+            MoveToColumn(column) => self.move_to_column(*column)?,
+            MoveDown(amount) => self.move_down(*amount)?,
+            MoveRight(amount) => self.move_right(*amount)?,
+            MoveLeft(amount) => self.move_left(*amount)?,
+            MoveToNextLine => self.move_to_next_line()?,
+            PrintText { content, style } => self.print_text(content, style)?,
+            ClearScreen => self.clear_screen()?,
+            SetColors(colors) => self.set_colors(*colors)?,
+            SetBackgroundColor(color) => self.set_background_color(*color)?,
+            Flush => self.flush()?,
+            PrintImage { image, options } => self.print_image(image, options)?,
+        };
+        Ok(())
+    }
+
+    fn cursor_row(&self) -> u16 {
+        self.cursor_row
+    }
+}
+
 impl<I: TerminalWrite> Drop for Terminal<I> {
     fn drop(&mut self) {
         self.writer.deinit();
     }
 }
 
-#[derive(Clone, Debug)]
-pub(crate) struct TextProperties {
-    pub(crate) height: u8,
-}
-
-impl Default for TextProperties {
-    fn default() -> Self {
-        Self { height: 1 }
-    }
-}
-
 pub(crate) fn should_hide_cursor() -> bool {
     // WezTerm on Windows fails to display images if we've hidden the cursor so we **always** hide it
     // unless we're on WezTerm on Windows.
